@@ -3,7 +3,7 @@
  *
  * Derived from Linux version by Richard Tobin.
  *
- * $Id: bsd.c,v 1.24 2002/11/10 20:29:30 jerdfelt Exp $
+ * $Id: bsd.c,v 1.28 2003/12/30 01:12:38 jerdfelt Exp $
  * $Name:  $
  *
  * This library is covered by the LGPL, read LICENSE for details.
@@ -43,9 +43,11 @@
 #endif
 
 #ifdef HAVE_OLD_DEV_USB_USB_H
-/* It appears some of the BSD's (OpenBSD atleast) have switched over to a */
-/* new naming convention, so we setup some macro's for backward */
-/* compability with older versions --jerdfelt */
+/*
+ * It appears some of the BSD's (OpenBSD atleast) have switched over to a
+ * new naming convention, so we setup some macro's for backward
+ * compability with older versions --jerdfelt
+ */
 
 /* struct usb_ctl_request */
 #define ucr_addr		addr
@@ -195,7 +197,7 @@ int usb_set_configuration(usb_dev_handle *dev, int configuration)
 
   ret = ioctl(dev->fd, USB_SET_CONFIG, &configuration);
   if (ret < 0)
-    USB_ERROR_STR(ret, "could not set config %d: %s", configuration,
+    USB_ERROR_STR(-errno, "could not set config %d: %s", configuration,
                   strerror(errno));
 
   dev->config = configuration;
@@ -232,7 +234,7 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
 
   ret = ioctl(dev->fd, USB_SET_ALTINTERFACE, &intf);
   if (ret < 0)
-    USB_ERROR_STR(ret, "could not set alt intf %d/%d: %s",
+    USB_ERROR_STR(-errno, "could not set alt intf %d/%d: %s",
                   dev->interface, alternate, strerror(errno));
 
   dev->altsetting = alternate;
@@ -265,7 +267,7 @@ static int ensure_ep_open(usb_dev_handle *dev, int ep, int mode)
     if (fd < 0 && errno == ENXIO)
       fd = open(buf, mode);
     if (fd < 0)
-      USB_ERROR_STR(fd, "can't open %s for bulk read: %s",
+      USB_ERROR_STR(-errno, "can't open %s for bulk read: %s",
                     buf, strerror(errno));
     info->ep_fd[ep] = fd;
   }
@@ -295,22 +297,22 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   ret = ioctl(fd, USB_SET_TIMEOUT, &timeout);
   if (ret < 0)
-    USB_ERROR_STR(ret, "error setting timeout: %s",
+    USB_ERROR_STR(-errno, "error setting timeout: %s",
                   strerror(errno));
 
   do {
     ret = write(fd, bytes+sent, size-sent);
     if (ret < 0)
 #if __FreeBSD__
-      USB_ERROR_STR(ret, "error writing to bulk endpoint %s.%d: %s",
+      USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%d: %s",
                     dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
-      USB_ERROR_STR(ret, "error writing to bulk endpoint %s.%02d: %s",
+      USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%02d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #endif
 
     sent += ret;
-  } while(ret > 0 && sent < size);
+  } while (ret > 0 && sent < size);
 
   return sent;
 }
@@ -337,22 +339,106 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   ret = ioctl(fd, USB_SET_TIMEOUT, &timeout);
   if (ret < 0)
-    USB_ERROR_STR(ret, "error setting timeout: %s",
-                  strerror(errno));
+    USB_ERROR_STR(-errno, "error setting timeout: %s", strerror(errno));
 
   ret = ioctl(fd, USB_SET_SHORT_XFER, &one);
   if (ret < 0)
-    USB_ERROR_STR(ret, "error setting short xfer: %s",
-                  strerror(errno));
+    USB_ERROR_STR(-errno, "error setting short xfer: %s", strerror(errno));
 
   do {
     ret = read(fd, bytes+retrieved, size-retrieved);
     if (ret < 0)
 #if __FreeBSD__
-      USB_ERROR_STR(ret, "error reading from bulk endpoint %s.%d: %s",
+      USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%d: %s",
                     dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
-      USB_ERROR_STR(ret, "error reading from bulk endpoint %s.%02d: %s",
+      USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%02d: %s",
+                  dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#endif
+    retrieved += ret;
+  } while (ret > 0 && retrieved < size);
+
+  return retrieved;
+}
+
+int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes, int size,
+                        int timeout)
+{
+  int fd, ret, sent = 0;
+
+  /* Ensure the endpoint address is correct */
+  ep &= ~USB_ENDPOINT_IN;
+
+  fd = ensure_ep_open(dev, ep, O_WRONLY);
+  if (fd < 0) {
+    if (usb_debug >= 2) {
+#if __FreeBSD__
+      fprintf (stderr, "usb_interrupt_write: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
+#else
+      fprintf (stderr, "usb_interrupt_write: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
+#endif
+    }
+    return fd;
+  }
+
+  ret = ioctl(fd, USB_SET_TIMEOUT, &timeout);
+  if (ret < 0)
+    USB_ERROR_STR(-errno, "error setting timeout: %s",
+                  strerror(errno));
+
+  do {
+    ret = write(fd, bytes+sent, size-sent);
+    if (ret < 0)
+#if __FreeBSD__
+      USB_ERROR_STR(-errno, "error writing to interrupt endpoint %s.%d: %s",
+                    dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#else
+      USB_ERROR_STR(-errno, "error writing to interrupt endpoint %s.%02d: %s",
+                  dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#endif
+
+    sent += ret;
+  } while (ret > 0 && sent < size);
+
+  return sent;
+}
+
+int usb_interrupt_read(usb_dev_handle *dev, int ep, char *bytes, int size,
+                       int timeout)
+{
+  int fd, ret, retrieved = 0, one = 1;
+
+  /* Ensure the endpoint address is correct */
+  ep |= USB_ENDPOINT_IN;
+
+  fd = ensure_ep_open(dev, ep, O_RDONLY);
+  if (fd < 0) {
+    if (usb_debug >= 2) {
+#if __FreeBSD__
+      fprintf (stderr, "usb_interrupt_read: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
+#else
+      fprintf (stderr, "usb_interrupt_read: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
+#endif
+    }
+    return fd;
+  }
+
+  ret = ioctl(fd, USB_SET_TIMEOUT, &timeout);
+  if (ret < 0)
+    USB_ERROR_STR(-errno, "error setting timeout: %s", strerror(errno));
+
+  ret = ioctl(fd, USB_SET_SHORT_XFER, &one);
+  if (ret < 0)
+    USB_ERROR_STR(-errno, "error setting short xfer: %s", strerror(errno));
+
+  do {
+    ret = read(fd, bytes+retrieved, size-retrieved);
+    if (ret < 0)
+#if __FreeBSD__
+      USB_ERROR_STR(-errno, "error reading from interrupt endpoint %s.%d: %s",
+                    dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#else
+      USB_ERROR_STR(-errno, "error reading from interrupt endpoint %s.%02d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #endif
     retrieved += ret;
@@ -386,12 +472,12 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
 #else
   if (ret < 0)
 #endif
-    USB_ERROR_STR(ret, "error setting timeout: %s",
+    USB_ERROR_STR(-errno, "error setting timeout: %s",
                   strerror(errno));
 
   ret = ioctl(dev->fd, USB_DO_REQUEST, &req);
   if (ret < 0)
-    USB_ERROR_STR(ret, "error sending control message: %s",
+    USB_ERROR_STR(-errno, "error sending control message: %s",
                   strerror(errno));
 
   return UGETW(req.ucr_request.wLength);
