@@ -69,7 +69,7 @@ int main(void)
   printf("bus/device  idVendor/idProduct\n");
   for (bus = usb_busses; bus; bus = bus->next) {
     for (dev = bus->devices; dev; dev = dev->next) {
-      int i, ret;
+      int ret, i;
       char string[256];
       usb_dev_handle *udev;
 
@@ -101,6 +101,8 @@ int main(void)
           else
             printf("- Unable to fetch serial number string\n");
         }
+
+	usb_close (udev);
       }
 
       if (!dev->config) {
